/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jz.dmp.cmdexectool.scheduler.common.task.sql;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.common.utils.EncryptionUtils;
import com.jz.dmp.cmdexectool.common.utils.FreeMarkerUtils;
import com.jz.dmp.cmdexectool.controller.bean.DmpProjectConfigInfoDto;
import com.jz.dmp.cmdexectool.entity.DmpSyncingDatasource;
import com.jz.dmp.cmdexectool.mapper.DmpSyncingDatasourceDao;
import com.jz.dmp.cmdexectool.scheduler.common.enums.DbType;
import com.jz.dmp.cmdexectool.scheduler.common.enums.MyDbType;
import com.jz.dmp.cmdexectool.scheduler.common.process.ResourceInfo;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;
import com.jz.dmp.cmdexectool.scheduler.common.utils.ParameterUtils;

/**
 * Sql/Hql parameter
 */
public class SqlParameters extends AbstractParameters {
	
	/**
	 * shell script
	 */
	private String script;

	private String taskAppId;
	
	/**
	* 数据源相关配置
	*/
	private String source;

	/**
	* 目标相关配置
	*/
	private String sink;
	
	/**
	* 环境相关配置
	*/
	private String env;
	
	/**
	* ETL相关配置
	*/
	private String transform;
	
	/**
	 * waterdropScript脚本
	 */
	private String waterdropScript;
	
	/**
	 * 前置语句
	 */
	private List<String> preStatements;
	
	/**
	 * 后置语句
	 */
	private List<String> posStatements;
	
	/**
	 * jdbcUrl
	 */
	private String jdbcUrl;
	
	/**
	 * jdbc user
	 */
	private String user;
	
	/**
	 * jdbc password
	 */
	private String password;
	
	/**
	 * 数据源类型
	 */
	private MyDbType myDbType;

	/**
	 * resource list
	 */
	private List<ResourceInfo> resourceList;

	public SqlParameters(String script, DmpProjectConfigInfoDto projectConfigInfoDto, DmpSyncingDatasourceDao dmpSyncingDatasourceDao, FreeMarkerConfigurer freeMarkerConfig, String publicKey) {
		source = "";
		env = "";
		sink = "";
		transform = "";
		
		this.script = script;
		
		JSONObject scriptObj = JSONObject.parseObject(script);
		String outputType = scriptObj.getString("outputType");
		
		String sqlScript = scriptObj.getString("sqlScript");
		
		//evn
		Map<String, String> envModel = new HashMap<String, String>();
		envModel.put("sparkappname", "Waterdrop");
		env = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_ENV, envModel, freeMarkerConfig);
		//source
		Integer sourceId = scriptObj.getInteger("sourceId");
		DmpSyncingDatasource dmpSyncingDatasource = dmpSyncingDatasourceDao.queryById(sourceId);
		
		this.jdbcUrl = dmpSyncingDatasource.getJdbcUrl();
		this.user = dmpSyncingDatasource.getUserName();
		this.password = EncryptionUtils.decode(dmpSyncingDatasource.getPassword(), publicKey);
		this.myDbType = MyDbType.obtainByIdStr(dmpSyncingDatasource.getId().toString());
		
		String sourceTableNames = scriptObj.getString("sourceTableNames");
		String[] tableNameArr = sourceTableNames.split(",");
		for (String tableName : tableNameArr) {
			Map<String, String> jdbcModel = new HashMap<String, String>();
			jdbcModel.put("driver", dmpSyncingDatasource.getDriverClassName());
			jdbcModel.put("url", this.jdbcUrl);
			jdbcModel.put("table", tableName);
			jdbcModel.put("result_table_name", tableName);
			jdbcModel.put("user", this.user);
			jdbcModel.put("password", this.password);
			source = source + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SOURCE_JDBC, jdbcModel, freeMarkerConfig);
		}
		
		if (CommConstant.OUTPUT_TYPE_CONSOLE.equals(outputType)) {
			//transform
			Map<String, String> transformSqlModel = new HashMap<String, String>();
			transformSqlModel.put("sql", sqlScript);
			transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
			//sink
			Map<String, String> stdoutModel = new HashMap<String, String>();
			sink = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SINK_CONSOLE, stdoutModel, freeMarkerConfig);
		}else if (CommConstant.OUTPUT_TYPE_HDFS.equals(outputType)) {
			//transform
			Map<String, String> transformSqlModel = new HashMap<String, String>();
			transformSqlModel.put("sql", sqlScript);
			transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
			//sink
			JSONObject hdfsObj = scriptObj.getJSONObject("hdfs");
			String hdfsDir = hdfsObj.getString("hdfsDir");
			
			Map<String, String> hdfsModel = new HashMap<String, String>();
			hdfsModel.put("path", hdfsDir);
			sink = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SINK_HDFS, hdfsModel, freeMarkerConfig);
		}else if (CommConstant.OUTPUT_TYPE_TABLE.equals(outputType)) {
			//transform
			Map<String, String> transformSqlModel = new HashMap<String, String>();
			transformSqlModel.put("sql", sqlScript);
			transformSqlModel.put("table_name", "t_view");
			transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
			
			JSONObject tableObj = scriptObj.getJSONObject("table");
			String tableFieldsObj = tableObj.getString("tableFields");
			String sqlStr = ParameterUtils.columnMappingHandler(tableFieldsObj);
			
			Map<String, String> transformMappingSqlModel = new HashMap<String, String>();
			transformSqlModel.put("sql", sqlStr);
			transform = transform + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_JSON2, transformMappingSqlModel, freeMarkerConfig);
			
			//sink
			
			//targetSource
			Integer targetSourceId = tableObj.getInteger("targetSourceId");
			DmpSyncingDatasource targetSource = dmpSyncingDatasourceDao.queryById(targetSourceId);
			
			Map<String, String> sinkJdbcModel = new HashMap<String, String>();
			sinkJdbcModel.put("save_mode", "overwrite");
			sinkJdbcModel.put("truncate", "true");
			sinkJdbcModel.put("url", targetSource.getJdbcUrl());
			sinkJdbcModel.put("driver", targetSource.getDriverClassName());
			sinkJdbcModel.put("user", targetSource.getUserName());
			sinkJdbcModel.put("password", targetSource.getPassword());
			sinkJdbcModel.put("dbtable", targetSource.getDbName());
			sink = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SINK_JDBC, sinkJdbcModel, freeMarkerConfig);
			
		}else if (CommConstant.OUTPUT_TYPE_TOPIC.equals(outputType)) {
			//transform
			Map<String, String> transformSqlModel = new HashMap<String, String>();
			transformSqlModel.put("sql", sqlScript);
			transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
			//sink
			
			JSONObject topicObj = scriptObj.getJSONObject("topic");
			
			Map<String, String> kafkaModel = new HashMap<String, String>();
			kafkaModel.put("topic", topicObj.getString("topic"));
			kafkaModel.put("broker", topicObj.getString("server"));
			sink = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SINK_KAFKA, kafkaModel, freeMarkerConfig);
		}else if (CommConstant.OUTPUT_TYPE_API.equals(outputType)) {
			//transform
			Map<String, String> transformSqlModel = new HashMap<String, String>();
			transformSqlModel.put("sql", sqlScript);
			transformSqlModel.put("table_name", "t_view");
			transform = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, transformSqlModel, freeMarkerConfig);
			
			JSONObject apiObj = scriptObj.getJSONObject("api");
			String columnFieldsObj = apiObj.getString("columnFields");
			String sqlStr = ParameterUtils.columnMappingHandler(columnFieldsObj);
			
			Map<String, String> transformMappingSqlModel = new HashMap<String, String>();
			transformSqlModel.put("sql", sqlStr);
			transform = transform + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_JSON2, transformMappingSqlModel, freeMarkerConfig);
			
			//sink
			Map<String, String> sinkApiModel = new HashMap<String, String>();
			sinkApiModel.put("url", apiObj.getString("apiUrl"));
			sinkApiModel.put("apiKey", apiObj.getString("apiKey"));
			sinkApiModel.put("method", apiObj.getString("method"));
			sinkApiModel.put("signType", apiObj.getString("signType"));
			sinkApiModel.put("authCode", apiObj.getString("authCode"));
			sink = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SINK_API, sinkApiModel, freeMarkerConfig);
		}
		//waterdrop script
		Map<String, String> waterdropModel = new HashMap<String, String>();
		waterdropModel.put("env", env);
		waterdropModel.put("source", source);
		waterdropModel.put("transform", transform);
		waterdropModel.put("sink", sink);
		this.waterdropScript = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL, waterdropModel, freeMarkerConfig);
		
		preStatements = new ArrayList<String>();
		preStatements.add("insert into test(id, name) values(1, 'test')");
		posStatements = new ArrayList<String>();
		posStatements.add("insert into test(id, name) values(2, 'test2')");
		
	}

	public String getScript() {
		return script;
	}

	public void setScript(String script) {
		this.script = script;
	}

	public String getTaskAppId() {
		return taskAppId;
	}

	public void setTaskAppId(String taskAppId) {
		this.taskAppId = taskAppId;
	}
	
	public String getSource() {
		return source;
	}

	public void setSource(String source) {
		this.source = source;
	}

	public String getSink() {
		return sink;
	}

	public void setSink(String sink) {
		this.sink = sink;
	}

	public String getEnv() {
		return env;
	}

	public void setEnv(String env) {
		this.env = env;
	}

	public String getTransform() {
		return transform;
	}

	public void setTransform(String transform) {
		this.transform = transform;
	}

	public String getWaterdropScript() {
		return waterdropScript;
	}

	public void setWaterdropScript(String waterdropScript) {
		this.waterdropScript = waterdropScript;
	}

	public List<String> getPreStatements() {
		return preStatements;
	}

	public void setPreStatements(List<String> preStatements) {
		this.preStatements = preStatements;
	}

	public List<String> getPosStatements() {
		return posStatements;
	}

	public void setPosStatements(List<String> posStatements) {
		this.posStatements = posStatements;
	}

	public List<ResourceInfo> getResourceList() {
		return resourceList;
	}

	public void setResourceList(List<ResourceInfo> resourceList) {
		this.resourceList = resourceList;
	}

	public String getJdbcUrl() {
		return jdbcUrl;
	}

	public void setJdbcUrl(String jdbcUrl) {
		this.jdbcUrl = jdbcUrl;
	}

	public String getUser() {
		return user;
	}

	public void setUser(String user) {
		this.user = user;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public MyDbType getMyDbType() {
		return myDbType;
	}

	public void setMyDbType(MyDbType myDbType) {
		this.myDbType = myDbType;
	}

	@Override
	public boolean checkParameters() {
		return waterdropScript != null && !waterdropScript.isEmpty();
	}

	@Override
	public List<ResourceInfo> getResourceFilesList() {
		return resourceList;
	}

}
