/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jz.dmp.cmdexectool.scheduler.server.worker.task.sql;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.nio.file.attribute.FileAttribute;
import java.nio.file.attribute.PosixFilePermission;
import java.nio.file.attribute.PosixFilePermissions;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.springframework.util.CollectionUtils;

import com.jz.dmp.cmdexectool.scheduler.common.Constants;
import com.jz.dmp.cmdexectool.scheduler.common.enums.DbType;
import com.jz.dmp.cmdexectool.scheduler.common.process.Property;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;
import com.jz.dmp.cmdexectool.scheduler.common.task.sql.SqlBinds;
import com.jz.dmp.cmdexectool.scheduler.common.task.sql.SqlParameters;
import com.jz.dmp.cmdexectool.scheduler.common.utils.OSUtils;
import com.jz.dmp.cmdexectool.scheduler.common.utils.ParameterUtils;
import com.jz.dmp.cmdexectool.scheduler.dao.datasource.BaseDataSource;
import com.jz.dmp.cmdexectool.scheduler.dao.utils.DatabaseUtils;
import com.jz.dmp.cmdexectool.scheduler.server.entity.TaskExecutionContext;
import com.jz.dmp.cmdexectool.scheduler.server.utils.ParamUtils;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.AbstractTask;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.CommandExecuteResult;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.WaterdropCommandExecutor;

/**
 * sql task
 */
public class SqlTask extends AbstractTask {

	/**
	 * sql parameters
	 */
	private SqlParameters sqlParameters;

	/**
	 * waterdrop command executor
	 */
	private WaterdropCommandExecutor waterdropCommandExecutor;

	/**
	 * taskExecutionContext
	 */
	private TaskExecutionContext taskExecutionContext;

	/**
	 * constructor
	 * 
	 * @param taskExecutionContext taskExecutionContext
	 * @param logger               logger
	 */
	public SqlTask(TaskExecutionContext taskExecutionContext, Logger logger) {
		super(taskExecutionContext, logger);

		this.taskExecutionContext = taskExecutionContext;
		this.waterdropCommandExecutor = new WaterdropCommandExecutor(this::logHandle, taskExecutionContext, logger);
	}

	@Override
	public void init() {
		logger.info("sql task params {}", taskExecutionContext.getTaskParams());

		//sqlParameters = JSONUtils.parseObject(taskExecutionContext.getTaskParams(), SqlParameters.class);
		sqlParameters = (SqlParameters) taskExecutionContext.getParameters();

		if (!sqlParameters.checkParameters()) {
			throw new RuntimeException("sql task params is not valid");
		}
	}

	@Override
	public void handle() throws Exception {
		try {
			
			BaseDataSource baseDataSource = new BaseDataSource() {
				
				@Override
				public String driverClassSelector() {
					// TODO Auto-generated method stub
					return null;
				}
				
				@Override
				public DbType dbTypeSelector() {
					// TODO Auto-generated method stub
					return null;
				}
			};
			
			baseDataSource.setDbType(sqlParameters.getMyDbType().getDbType().name());
			baseDataSource.setUser(sqlParameters.getUser());
			baseDataSource.setPassword(sqlParameters.getPassword());
			baseDataSource.setAddress(sqlParameters.getJdbcUrl());
			
			List<SqlBinds> preStatementSqlBinds = Optional.ofNullable(sqlParameters.getPreStatements())
                    .orElse(new ArrayList<>())
                    .stream()
                    .map(DatabaseUtils::getSqlAndSqlParamsMap)
                    .collect(Collectors.toList());
            
            
            List<SqlBinds> postStatementSqlBinds = Optional.ofNullable(sqlParameters.getPosStatements())
                    .orElse(new ArrayList<>())
                    .stream()
                    .map(DatabaseUtils::getSqlAndSqlParamsMap)
                    .collect(Collectors.toList());
			
			//判断是否需要运行前置sql
            if (!CollectionUtils.isEmpty(preStatementSqlBinds)) {
            	DatabaseUtils.executeUpdateSql(preStatementSqlBinds, baseDataSource);
			}
			
			// construct process
			CommandExecuteResult commandExecuteResult = waterdropCommandExecutor.run(buildCommand());
			
			//判断是否运行后置sql
			if (!CollectionUtils.isEmpty(postStatementSqlBinds)) {
            	DatabaseUtils.executeUpdateSql(postStatementSqlBinds, baseDataSource);
			}
			
			setExitStatusCode(commandExecuteResult.getExitStatusCode());
			setAppIds(commandExecuteResult.getAppIds());
			setProcessId(commandExecuteResult.getProcessId());
		} catch (Exception e) {
			logger.error("sql task error", e);
			setExitStatusCode(Constants.EXIT_CODE_FAILURE);
			throw e;
		}
	}
	
	
	@Override
	public void cancelApplication(boolean cancelApplication) throws Exception {
		// cancel process
		waterdropCommandExecutor.cancelApplication();
	}

	/**
	 * create command
	 * 
	 * @return file name
	 * @throws Exception exception
	 */
	private String buildCommand() throws Exception {
		// generate scripts
		String fileName = String.format("%s/%s_node.%s", taskExecutionContext.getExecutePath(),
				taskExecutionContext.getTaskAppId(), OSUtils.isWindows() ? "bat" : "sh");

		Path path = new File(fileName).toPath();

		if (Files.exists(path)) {
			return fileName;
		}

		logger.info("raw script : {}", sqlParameters.getScript());
		logger.info("task execute path : {}", taskExecutionContext.getExecutePath());

		Set<PosixFilePermission> perms = PosixFilePermissions.fromString(Constants.RWXR_XR_X);
		FileAttribute<Set<PosixFilePermission>> attr = PosixFilePermissions.asFileAttribute(perms);

		if (OSUtils.isWindows()) {
			// Files.createFile(path);
			File file = path.toFile();
			File parentFile = file.getParentFile();
			if (!parentFile.exists()) {
				parentFile.mkdirs();
			}
			file.createNewFile();
		} else {
			Files.createFile(path, attr);
		}

		Files.write(path, sqlParameters.getWaterdropScript().getBytes(), StandardOpenOption.APPEND);

		return fileName;
	}

	@Override
	public AbstractParameters getParameters() {
		return sqlParameters;
	}

}
